@@ -2,47 +2,27 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
 
-import compas
+import Rhino
 import compas_rhino
 
 from compas.datastructures import Mesh
-from compas_rhino.geometry._geometry import RhinoGeometry
-# from compas.geometry import subtract_vectors
-# from compas.geometry import angle_vectors
-# from compas.geometry import distance_point_point
 from compas.utilities import geometric_key
 
-try:
-    import Rhino
-except ImportError:
-    compas.raise_if_ironpython()
+from compas_rhino.geometry._geometry import BaseRhinoGeometry
 
 
 __all__ = ['RhinoSurface']
 
 
-class RhinoSurface(RhinoGeometry):
-    """"""
+class RhinoSurface(BaseRhinoGeometry):
+    """Wrapper for Rhino surface objects."""
 
     def __init__(self):
         super(RhinoSurface, self).__init__()
 
     @classmethod
-    def from_guid(cls, guid):
-        obj = compas_rhino.find_object(guid)
-        surf = cls()
-        surf.guid = guid
-        surf.object = obj
-        surf.geometry = obj.Geometry
-        return surf
-
-    @classmethod
-    def from_object(cls, obj):
-        surf = cls()
-        surf.guid = obj.Id
-        surf.object = obj
-        surf.geometry = obj.Geometry
-        return surf
+    def from_geometry(cls):
+        raise NotImplementedError
 
     @classmethod
     def from_selection(cls):
@@ -53,10 +33,22 @@ class RhinoSurface(RhinoGeometry):
         raise NotImplementedError
 
     def brep_to_compas(self, cls=None):
+        """Convert the surface b-rep loops to a COMPAS mesh.
+
+        Parameters
+        ----------
+        cls : :class:`compas.datastructures.Mesh`, optional
+            The type of COMPAS mesh.
+
+        Returns
+        -------
+        :class:`compas.datastructures.Mesh`
+            The resulting mesh.
+
+        """
         if not self.geometry.HasBrepForm:
             return
         brep = Rhino.Geometry.Brep.TryConvertBrep(self.geometry)
-
         gkey_xyz = {}
         faces = []
         for loop in brep.Loops:
@@ -77,19 +69,56 @@ class RhinoSurface(RhinoGeometry):
                 face.append(ep_gkey)
                 gkey_xyz[ep_gkey] = ep
             faces.append(face)
-
         gkey_index = {gkey: index for index, gkey in enumerate(gkey_xyz)}
         vertices = [list(xyz) for gkey, xyz in gkey_xyz.items()]
         faces = [[gkey_index[gkey] for gkey in f] for f in faces]
+        polygons = []
+        for temp in faces:
+            face = []
+            for vertex in temp:
+                if vertex not in face:
+                    face.append(vertex)
+            polygons.append(face)
         cls = cls or Mesh
-        return cls.from_vertices_and_faces(vertices, faces)
+        return cls.from_vertices_and_faces(vertices, polygons)
 
-    # move to compas.datastructures.Mesh.from_meshgrid(RhinoSurface.to_meshgrid(over_space=True))?
     def uv_to_compas(self, cls=None, density=(10, 10)):
+        """Convert the surface UV space to a COMPAS mesh.
+
+        Parameters
+        ----------
+        cls : :class:`compas.datastructures.Mesh`, optional
+            The type of mesh.
+        density : tuple of int, optional
+            The density in the U and V directions.
+            Default is ``u = 10`` and ``v = 10``.
+
+        Returns
+        -------
+        :class:`compas.datastructures.Mesh`
+            The COMPAS mesh.
+        """
         return self.heightfield_to_compas(cls=cls, density=density, over_space=True)
 
-    # move to compas.datastructures.Mesh.from_meshgrid(RhinoSurface.to_meshgrid(over_space=False))?
     def heightfield_to_compas(self, cls=None, density=(10, 10), over_space=False):
+        """Convert a heightfiled of the surface to a COMPAS mesh.
+
+        Parameters
+        ----------
+        cls : :class:`compas.datastructures.Mesh`, optional
+            The type of mesh.
+        density : tuple of int, optional
+            The density in the two grid directions.
+            Default is ``u = 10`` and ``v = 10``.
+        over_space : bool, optional
+            Construct teh grid over the surface UV space instead of the XY axes.
+            Default is ``False``.
+
+        Returns
+        -------
+        :class:`compas.datastructures.Mesh`
+            The COMPAS mesh.
+        """
         try:
             u, v = density
         except Exception:
@@ -110,10 +139,22 @@ class RhinoSurface(RhinoGeometry):
     #
     # ==========================================================================
 
-    def space(self, density=10):
+    def space(self, density=(10, 10)):
+        """Construct a parameter grid overt the UV space of the surface.
+
+        Parameters
+        ----------
+        density : tuple, optional
+            The density in the U and V directions of the parameter space.
+            Default is ``10`` in both directions.
+
+        Returns
+        -------
+        list
+            A list of UV parameter tuples.
+        """
         rs = compas_rhino.rs
         rs.EnableRedraw(False)
-
         try:
             du, dv = density
         except TypeError:
@@ -121,14 +162,12 @@ class RhinoSurface(RhinoGeometry):
             dv = density
         density_u = int(du)
         density_v = int(dv)
-
         if rs.IsPolysurface(self.guid):
             faces = rs.ExplodePolysurfaces(self.guid)
         elif rs.IsSurface(self.guid):
             faces = [self.guid]
         else:
             raise Exception('Object is not a surface.')
-
         uv = []
         for face in faces:
             domain_u = rs.SurfaceDomain(face, 0)
@@ -139,19 +178,31 @@ class RhinoSurface(RhinoGeometry):
             for i in range(density_u):
                 for j in range(density_v):
                     uv.append((domain_u[0] + i * du, domain_v[0] + j * dv))
-
         if len(faces) > 1:
             rs.DeleteObjects(faces)
         rs.EnableRedraw(True)
         return uv
 
-    # not taking the heighfield over the space is only possible
-    # if the surface is in fact a height field
-    # split up and rename!
-    def heightfield(self, density=10, over_space=True):
+    def heightfield(self, density=(10, 10), over_space=True):
+        """Construct a point grid over the surface.
+
+        Parameters
+        ----------
+        density : tuple, optional
+            The density in the U and V directions of the grid.
+            Default is ``10`` in both directions.
+        over_space : bool, optional
+            Construct the grid over the UV space of the surface.
+            Default is ``True``.
+
+        Returns
+        -------
+        list
+            List of grid points.
+
+        """
         rs = compas_rhino.rs
         rs.EnableRedraw(False)
-
         try:
             du, dv = density
         except TypeError:
@@ -159,14 +210,12 @@ class RhinoSurface(RhinoGeometry):
             dv = density
         du = int(du)
         dv = int(dv)
-
         if rs.IsPolysurface(self.guid):
             faces = rs.ExplodePolysurfaces(self.guid)
         elif rs.IsSurface(self.guid):
             faces = [self.guid]
         else:
             raise Exception('Object is not a surface.')
-
         xyz = []
         if over_space:
             for guid in faces:
@@ -190,7 +239,6 @@ class RhinoSurface(RhinoGeometry):
                         seeds.append(seed)
                 points = map(list, rs.ProjectPointToSurface(seeds, guid, [0, 0, 1]))
                 xyz += points
-
         if len(faces) > 1:
             rs.DeleteObjects(faces)
         rs.EnableRedraw(True)
@@ -200,11 +248,8 @@ class RhinoSurface(RhinoGeometry):
     #     """"""
     #     if not points:
     #         points = self.heightfield()
-
     #     tol = rs.UnitAbsoluteTolerance()
-
     #     descent = []
-
     #     if rs.IsPolysurface(self.guid):
     #         rs.EnableRedraw(False)
     #         faces = {}
@@ -235,16 +280,13 @@ class RhinoSurface(RhinoGeometry):
     #             descent.append((p0, vector))
     #     else:
     #         raise Exception('Object is not a surface.')
-
     #     return descent
 
     # def curvature(self, points=None):
     #     """"""
     #     if not points:
     #         points = self.heightfield()
-
     #     curvature = []
-
     #     if rs.IsPolysurface(self.guid):
     #         rs.EnableRedraw(False)
     #         faces = {}
@@ -269,7 +311,6 @@ class RhinoSurface(RhinoGeometry):
     #             curvature.append((point, (props[1], props[3], props[5])))
     #     else:
     #         raise Exception('Object is not a surface.')
-
     #     return curvature
 
     # def borders(self, type=1):
@@ -288,7 +329,6 @@ class RhinoSurface(RhinoGeometry):
     #     -------
     #     list
     #         The GUIDs of the extracted border curves.
-
     #     """
     #     border = rs.DuplicateSurfaceBorder(self.guid, type=type)
     #     curves = rs.ExplodeCurves(border, delete_input=True)
@@ -301,67 +341,20 @@ class RhinoSurface(RhinoGeometry):
     #     -------
     #     list
     #         The list of XYZ coordinates of surface boundary kinks.
-
     #     """
     #     kinks = []
     #     borders = self.borders(type=0)
-
     #     for border in borders:
     #         border = RhinoCurve(border)
     #         extremities = map(lambda x: rs.EvaluateCurve(border.guid, rs.CurveParameter(border.guid, x)), [0., 1.])
-
     #         if border.is_closed():
     #             start_tgt, end_tgt = border.tangents(extremities)
     #             if angle_vectors(start_tgt, end_tgt) > threshold:
     #                 kinks += extremities
-
     #         else:
     #             kinks += extremities
-
     #     return list(set(kinks))
 
-    # def project_point(self, point, direction=(0, 0, 1)):
-    #     projections = rs.ProjectPointToSurface(point, self.guid, direction)
-    #     if not projections:
-    #         return self.closest_point(point)
-    #     return list(projections[0])
-
-    # def project_points(self, points, direction=(0, 0, 1), include_none=True):
-    #     projections = rs.ProjectPointToSurface(points, self.guid, direction)
-    #     if not projections:
-    #         return self.closest_points(points)
-    #     projections[:] = [self.closest_point(point) if not point else point for point in projections]
-    #     return map(list, projections)
-
-    # def pull_point(self, point):
-    #     pass
-
-    # def pull_points(self, points):
-    #     pass
-
-    # def pull_curve(self, curve):
-    #     pass
-
-    # def pull_curves(self, curves):
-    #     pass
-
-    # def pull_mesh(self, mesh, fixed=None, d=1.0):
-    #     if not fixed:
-    #         fixed = []
-    #     fixed = set(fixed)
-    #     for key, attr in mesh.vertices(True):
-    #         if key in fixed:
-    #             continue
-    #         xyz = mesh.vertex_coordinates(key)
-    #         point = self.closest_point(xyz)
-    #         dx, dy, dz = subtract_vectors(point, xyz)
-    #         mesh.vertex[key]['x'] += d * dx
-    #         mesh.vertex[key]['y'] += d * dy
-    #         mesh.vertex[key]['z'] += d * dz
-
-    # def pull_meshes(self, meshes):
-    #     pass
-
     def closest_point(self, xyz):
         """Return the XYZ coordinates of the closest point on the surface from input XYZ-coordinates.
 
@@ -404,109 +397,10 @@ class RhinoSurface(RhinoGeometry):
     # def closest_points_on_boundaries(self, points):
     #     return [self.closest_point_on_boundaries(point) for point in points]
 
-    # # --------------------------------------------------------------------------
-    # # mapping
-    # # --------------------------------------------------------------------------
-
-    # def point_xyz_to_uv(self, xyz):
-    #     """Return the UV point from the mapping of a XYZ point based on the UV parameterisation of the surface.
-
-    #     Parameters
-    #     ----------
-    #     xyz : list
-    #         (x, y, z) coordinates.
-
-    #     Returns
-    #     -------
-    #     list
-    #         The (u, v) coordinates of the mapped point.
-
-    #     """
-    #     return rs.SurfaceClosestPoint(self.guid, xyz)
-
-    # def point_uv_to_xyz(self, uv):
-    #     """Return the XYZ point from the inverse mapping of a UV point based on the UV parameterisation of the surface.
-
-    #     Parameters
-    #     ----------
-    #     uv : list
-    #         (u, v) coordinates.
-
-    #     Returns
-    #     -------
-    #     list
-    #         The (x, y, z) coordinates of the inverse-mapped point.
-
-    #     """
-    #     u, v = uv
-    #     return tuple(rs.EvaluateSurface(self.guid, *uv))
-
-    # def line_uv_to_xyz(self, line):
-    #     """Return the XYZ points from the inverse mapping of a UV line based on the UV parameterisation of the surface.
-
-    #     Parameters
-    #     ----------
-    #     uv : list
-    #         List of (u, v) coordinates.
-
-    #     Returns
-    #     -------
-    #     list
-    #         The list of XYZ coordinates of the inverse-mapped line.
-
-    #     """
-    #     return (self.point_uv_to_xyz(line[0]), self.point_uv_to_xyz(line[1]))
-
-    # def polyline_uv_to_xyz(self, polyline):
-    #     """Return the XYZ points from the inverse mapping of a UV polyline based on the UV parameterisation of the surface.
-
-    #     Parameters
-    #     ----------
-    #     uv : list
-    #         List of (u, v) coordinates.
-
-    #     Returns
-    #     -------
-    #     list
-    #         The list of (x, y, z) coordinates of the inverse-mapped polyline.
-
-    #     """
-    #     return [self.point_uv_to_xyz(vertex) for vertex in polyline]
-
-    # def mesh_uv_to_xyz(self, mesh):
-    #     """Return the mesh from the inverse mapping of a UV mesh based on the UV parameterisation of the surface.
-    #     The third coordinate of the mesh vertices is discarded.
-
-    #     Parameters
-    #     ----------
-    #     mesh : Mesh
-    #         A mesh.
-
-    #     Returns
-    #     -------
-    #     mesh : Mesh
-    #         The mesh once mapped back to the surface.
-
-    #     """
-
-    #     for vkey in mesh.vertices():
-    #         x, y, z = self.point_uv_to_xyz(mesh.vertex_coordinates(vkey)[:2])
-    #         mesh.vertex[vkey]['x'] = x
-    #         mesh.vertex[vkey]['y'] = y
-    #         mesh.vertex[vkey]['z'] = z
-    #     return mesh
-
 
 # ==============================================================================
 # Main
 # ==============================================================================
 
 if __name__ == '__main__':
-
-    surface = RhinoSurface.from_selection()
-
-    print(surface.guid)
-    print(surface.object)
-    print(surface.geometry)
-    print(surface.type)
-    print(surface.name)
+    pass
